import { Unit, Store, Event, Effect, Subscription } from 'effector'

interface StorageAdapter {
  <State>(key: string, update: (raw?: any) => any): {
    get(raw?: any, ctx?: any): State | Promise<State>
    set(value: State, ctx?: any): void
  }
  keyArea?: any
  noop?: boolean
}
interface StorageAdapterFactory<AdapterConfig> {
  (config?: AdapterConfig): StorageAdapter
  factory: true
}
type Contract<Data> =
  | ((raw: unknown) => raw is Data)
  | {
      isData: (raw: unknown) => raw is Data
      getErrorMessages: (raw: unknown) => string[]
    }
type Done<State> = {
  key: string
  keyPrefix: string
  operation: 'set' | 'get'
  value: State
}
type Fail<Err> = {
  key: string
  keyPrefix: string
  operation: 'set' | 'get'
  error: Err
  value?: any
}
type Finally<State, Err> =
  | (Done<State> & {
      status: 'done'
    })
  | (Fail<Err> & {
      status: 'fail'
    })
interface ConfigPersist {
  pickup?: Unit<any>
  context?: Unit<any>
  keyPrefix?: string
  contract?: Contract<any>
}
interface ConfigAdapter {
  adapter: StorageAdapter
}
interface ConfigAdapterFactory<AdapterConfig> {
  adapter: StorageAdapterFactory<AdapterConfig>
}
interface ConfigCommon<State, Err = Error> {
  clock?: Unit<any>
  done?: Unit<Done<State>>
  fail?: Unit<Fail<Err>>
  finally?: Unit<Finally<State, Err>>
  pickup?: Unit<any>
  context?: Unit<any>
  key?: string
  keyPrefix?: string
  contract?: Contract<State | undefined>
}
interface ConfigJustStore<State> {
  store: Store<State>
}
interface ConfigJustSourceTarget<State> {
  source: Store<State> | Event<State> | Effect<State, any, any>
  target: Store<State> | Event<State> | Effect<State, any, any>
}
interface ConfigStore<State, Err = Error>
  extends ConfigCommon<State, Err>,
    ConfigJustStore<State> {}
interface ConfigSourceTarget<State, Err = Error>
  extends ConfigCommon<State, Err>,
    ConfigJustSourceTarget<State> {}

/**
 * Main `persist` function
 */
declare function persist<State, Err = Error>(
  config: Partial<
    (ConfigAdapter | ConfigAdapterFactory<any>) &
      ConfigPersist &
      ConfigStore<State, Err> &
      ConfigSourceTarget<State, Err>
  >
): Subscription

export { persist }
